import { Directive, ElementRef, Renderer2, Input, NgModule } from '@angular/core';
import { Location, LocationStrategy, PathLocationStrategy } from '@angular/common';
import { isArray } from 'util';
import { NavigationStart, NavigationEnd, Router } from '@angular/router';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxRouteHighlighterConfig {
    /**
     * @param {?=} config
     */
    constructor(config) {
        this.config = config;
    }
    /**
     * @param {?} url
     * @param {?=} className
     * @return {?}
     */
    getNormalConfig(url, className) {
        this.config = {
            className: className ? className : 'active',
            links: url,
            strict: false
        };
        return new NgxRouteHighlighterConfig(this.config);
    }
    /**
     * @param {?} url
     * @param {?=} className
     * @return {?}
     */
    getStrictConfig(url, className) {
        this.config = {
            className: className ? className : 'active',
            links: url,
            strict: true
        };
        return new NgxRouteHighlighterConfig(this.config);
    }
}
if (false) {
    /** @type {?} */
    NgxRouteHighlighterConfig.prototype.config;
}
/**
 * @record
 */
function LinksEntity() { }
if (false) {
    /** @type {?} */
    LinksEntity.prototype.links;
    /** @type {?} */
    LinksEntity.prototype.className;
    /** @type {?|undefined} */
    LinksEntity.prototype.strict;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RouteHighlighterDirective {
    /**
     * @param {?} el
     * @param {?} location
     * @param {?} r2
     * @param {?} router
     */
    constructor(el, location, r2, router) {
        this.el = el;
        this.location = location;
        this.r2 = r2;
        this.router = router;
        this.defaultConfig = {
            className: 'active',
            links: null,
            strict: false
        };
        this.ngxRouteHighlighter = new NgxRouteHighlighterConfig(this.defaultConfig);
        this.activeClass = 'active';
        router.events.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            if (event instanceof NavigationStart) {
                console.log('navigation start and removed class');
                this.r2.removeClass(this.el.nativeElement, this.activeClass);
            }
            else if (event instanceof NavigationEnd) {
                this.setClass();
            }
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.ngxRouteHighlighter = new NgxRouteHighlighterConfig(this.defaultConfig);
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.ngxRouteHighlighter = changes.ngxRouteHighlighter.currentValue;
        console.log(this.ngxRouteHighlighter.config);
        if (this.ngxRouteHighlighter.config) {
            this.activeClass = this.ngxRouteHighlighter.config.className ? this.ngxRouteHighlighter.config.className : 'active';
        }
        else {
            this.activeClass = 'active';
        }
        // this.setClass();
    }
    /**
     * @return {?}
     */
    setClass() {
        if (this.ngxRouteHighlighter instanceof NgxRouteHighlighterConfig) {
            // console.log(this.ngxRouteHighlighter.config);
            if (isArray(this.ngxRouteHighlighter.config.links) && this.ngxRouteHighlighter.config.links !== null) {
                if (this.linkMatcher(this.ngxRouteHighlighter.config.links, this.ngxRouteHighlighter.config.strict)) {
                    // console.log('addingclass to')
                    this.r2.addClass(this.el.nativeElement, this.activeClass);
                }
                else {
                    // console.log('removing class to')
                    // this.r2.removeClass(this.el.nativeElement, this.activeClass);
                }
            }
            else {
                throw Error('ngx Route Highlighter Directive needs links as <array> of <String> where provided data is of TYPE => ' + typeof (this.ngxRouteHighlighter));
            }
        }
        else {
            throw Error('ngx Route Highlighter Directive needs <NgxRouteHighlighterConfig> where provided data is of TYPE => ' + typeof (this.ngxRouteHighlighter));
        }
    }
    /**
     * @param {?} links
     * @param {?=} strict
     * @return {?}
     */
    linkMatcher(links, strict) {
        /** @type {?} */
        let foundMatch = false;
        /** @type {?} */
        let temp = [];
        console.log('started machinig links');
        if (!strict) {
            strict = false;
        }
        // console.log(links)
        links.forEach((/**
         * @param {?} link
         * @return {?}
         */
        link => {
            if (strict) {
                if (this.location.path().includes(link)) {
                    foundMatch = true;
                    console.log(this.el.nativeElement, 'added');
                    temp.push(link);
                }
            }
            else {
                if (this.location.path() === link) {
                    foundMatch = true;
                    console.log(this.el.nativeElement, 'added');
                    temp.push(link);
                }
            }
        }));
        console.log('from', links, 'Matched Links are', temp);
        return foundMatch;
    }
}
RouteHighlighterDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxRouteHighlighter]',
                providers: [Location, { provide: LocationStrategy, useClass: PathLocationStrategy }],
            },] }
];
/** @nocollapse */
RouteHighlighterDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Location },
    { type: Renderer2 },
    { type: Router }
];
RouteHighlighterDirective.propDecorators = {
    ngxRouteHighlighter: [{ type: Input }],
    activeClass: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    RouteHighlighterDirective.prototype.defaultConfig;
    /** @type {?} */
    RouteHighlighterDirective.prototype.ngxRouteHighlighter;
    /** @type {?} */
    RouteHighlighterDirective.prototype.activeClass;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.el;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.location;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.r2;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.router;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxRouteHighlighterModule {
    /**
     * @param {?} router
     * @return {?}
     */
    static forRoot(router) {
        return {
            ngModule: NgxRouteHighlighterModule,
            imports: [
                router,
            ]
        };
    }
}
NgxRouteHighlighterModule.decorators = [
    { type: NgModule, args: [{
                declarations: [RouteHighlighterDirective],
                imports: [],
                exports: [RouteHighlighterDirective],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxRouteHighlighterConfig, NgxRouteHighlighterModule, RouteHighlighterDirective };
//# sourceMappingURL=ngx-route-highlighter.js.map
